@@ -1,7 +1,6 @@
-use std::{convert::TryFrom, path::Path, vec::IntoIter};
+use std::{convert::TryFrom, path::Path};
 
 use codemap::{CodeMap, Span, Spanned};
-use peekmore::{PeekMore, PeekMoreIterator};
 
 use crate::{
     atrule::{
@@ -12,6 +11,7 @@ use crate::{
     },
     builtin::modules::{ModuleConfig, Modules},
     error::SassResult,
+    lexer::Lexer,
     scope::{Scope, Scopes},
     selector::{
         ComplexSelectorComponent, ExtendRule, ExtendedSelector, Extender, Selector, SelectorParser,
@@ -63,18 +63,15 @@ pub(crate) enum Stmt {
     Import(String),
 }
 
-/// We could use a generic for the toks, but it makes the API
-/// much simpler to work with if it isn't generic. The performance
-/// hit (if there is one) is not important for now.
 // todo: merge at_root and at_root_has_selector into an enum
-pub(crate) struct Parser<'a> {
-    pub toks: &'a mut PeekMoreIterator<IntoIter<Token>>,
+pub(crate) struct Parser<'a, 'b> {
+    pub toks: &'a mut Lexer<'b>,
     pub map: &'a mut CodeMap,
     pub path: &'a Path,
     pub global_scope: &'a mut Scope,
     pub scopes: &'a mut Scopes,
     pub content_scopes: &'a mut Scopes,
-    pub super_selectors: &'a mut NeverEmptyVec<Selector>,
+    pub super_selectors: &'a mut NeverEmptyVec<ExtendedSelector>,
     pub span_before: Span,
     pub content: &'a mut Vec<Content>,
     pub flags: ContextFlags,
@@ -92,7 +89,7 @@ pub(crate) struct Parser<'a> {
     pub module_config: &'a mut ModuleConfig,
 }
 
-impl<'a> Parser<'a> {
+impl<'a, 'b> Parser<'a, 'b> {
     pub fn parse(&mut self) -> SassResult<Vec<Stmt>> {
         let mut stmts = Vec::new();
 
@@ -109,39 +106,43 @@ impl<'a> Parser<'a> {
             }
             self.at_root = true;
         }
+
         Ok(stmts)
     }
 
     pub fn expect_char(&mut self, c: char) -> SassResult<()> {
         match self.toks.peek() {
-            Some(Token { kind, pos }) if *kind == c => {
-                self.span_before = *pos;
+            Some(Token { kind, pos }) if kind == c => {
+                self.span_before = pos;
                 self.toks.next();
                 Ok(())
             }
-            Some(Token { pos, .. }) => Err((format!("expected \"{}\".", c), *pos).into()),
+            Some(Token { pos, .. }) => Err((format!("expected \"{}\".", c), pos).into()),
             None => Err((format!("expected \"{}\".", c), self.span_before).into()),
         }
     }
 
     pub fn consume_char_if_exists(&mut self, c: char) -> bool {
         if let Some(Token { kind, .. }) = self.toks.peek() {
-            if *kind == c {
+            if kind == c {
                 self.toks.next();
                 return true;
             }
         }
+
         false
     }
 
     pub fn expect_identifier(&mut self, ident: &'static str) -> SassResult<()> {
         let this_ident = self.parse_identifier_no_interpolation(false)?;
+
         self.span_before = this_ident.span;
+
         if this_ident.node == ident {
             return Ok(());
         }
 
-        Err((format!("Expected \"{}\".", ident), self.span_before).into())
+        Err((format!("Expected \"{}\".", ident), this_ident.span).into())
     }
 
     fn parse_stmt(&mut self) -> SassResult<Vec<Stmt>> {
@@ -150,7 +151,7 @@ impl<'a> Parser<'a> {
             if self.flags.in_function() && !stmts.is_empty() {
                 return Ok(stmts);
             }
-            self.span_before = *pos;
+            self.span_before = pos;
             match kind {
                 '@' => {
                     self.toks.next();
@@ -183,9 +184,8 @@ impl<'a> Parser<'a> {
                             if self.at_root {
                                 stmts.append(&mut self.parse_at_root()?);
                             } else {
-                                stmts.push(Stmt::AtRoot {
-                                    body: self.parse_at_root()?,
-                                });
+                                let body = self.parse_at_root()?;
+                                stmts.push(Stmt::AtRoot { body });
                             }
                         }
                         AtRuleKind::Error => {
@@ -206,12 +206,11 @@ impl<'a> Parser<'a> {
                                 span,
                             } = self.parse_value(false, &|_| false)?;
                             span.merge(kind_string.span);
-                            if let Some(Token { kind: ';', pos }) = self.toks.peek() {
-                                kind_string.span.merge(*pos);
-                                self.toks.next();
-                            }
+
+                            self.consume_char_if_exists(';');
+
                             self.warn(&Spanned {
-                                node: message.to_css_string(span)?,
+                                node: message.to_css_string(span, false)?,
                                 span,
                             });
                         }
@@ -221,10 +220,9 @@ impl<'a> Parser<'a> {
                                 span,
                             } = self.parse_value(false, &|_| false)?;
                             span.merge(kind_string.span);
-                            if let Some(Token { kind: ';', pos }) = self.toks.peek() {
-                                kind_string.span.merge(*pos);
-                                self.toks.next();
-                            }
+
+                            self.consume_char_if_exists(';');
+
                             self.debug(&Spanned {
                                 node: message.inspect(span)?,
                                 span,
@@ -243,10 +241,14 @@ impl<'a> Parser<'a> {
                                     .into());
                             }
 
-                            read_until_semicolon_or_closing_curly_brace(self.toks)?;
-                            if let Some(Token { kind: ';', .. }) = self.toks.peek() {
-                                self.toks.next();
+                            let val = self.parse_value(false, &|_| false)?;
+
+                            self.consume_char_if_exists(';');
+
+                            if !val.node.is_quoted_string() {
+                                return Err(("Expected string.", val.span).into());
                             }
+
                             continue;
                         }
                         AtRuleKind::Media => stmts.push(self.parse_media()?),
@@ -287,14 +289,14 @@ impl<'a> Parser<'a> {
                     }
                 }
                 '\u{0}'..='\u{8}' | '\u{b}'..='\u{1f}' => {
-                    return Err(("expected selector.", *pos).into())
+                    return Err(("expected selector.", pos).into())
                 }
                 '}' => {
                     self.toks.next();
                     break;
                 }
                 // dart-sass seems to special-case the error message here?
-                '!' | '{' => return Err(("expected \"}\".", *pos).into()),
+                '!' | '{' => return Err(("expected \"}\".", pos).into()),
                 _ => {
                     if self.flags.in_function() {
                         return Err((
@@ -356,14 +358,15 @@ impl<'a> Parser<'a> {
                                 .parse_selector(true, false, init)?
                                 .0
                                 .resolve_parent_selectors(
-                                    self.super_selectors.last(),
+                                    &self.super_selectors.last().clone().into_selector(),
                                     !at_root || self.at_root_has_selector,
                                 )?;
                             self.scopes.enter_new_scope();
-                            self.super_selectors.push(selector.clone());
 
                             let extended_selector = self.extender.add_selector(selector.0, None);
 
+                            self.super_selectors.push(extended_selector.clone());
+
                             let body = self.parse_stmt()?;
                             self.scopes.exit_scope();
                             self.super_selectors.pop();
@@ -403,9 +406,12 @@ impl<'a> Parser<'a> {
             span = span.merge(pos);
             match kind {
                 '#' => {
-                    if let Some(Token { kind: '{', .. }) = self.toks.peek().copied() {
-                        self.toks.next();
-                        string.push_str(&self.parse_interpolation()?.to_css_string(span)?);
+                    if self.consume_char_if_exists('{') {
+                        string.push_str(
+                            &self
+                                .parse_interpolation()?
+                                .to_css_string(span, self.options.is_compressed())?,
+                        );
                     } else {
                         string.push('#');
                     }
@@ -447,15 +453,13 @@ impl<'a> Parser<'a> {
             return Err(("expected \"{\".", span).into());
         }
 
-        // we must collect here because the parser isn't generic over the iterator
-        #[allow(clippy::needless_collect)]
         let sel_toks: Vec<Token> = string.chars().map(|x| Token::new(span, x)).collect();
 
-        let mut iter = sel_toks.into_iter().peekmore();
+        let mut lexer = Lexer::new(sel_toks);
 
         let selector = SelectorParser::new(
             &mut Parser {
-                toks: &mut iter,
+                toks: &mut lexer,
                 map: self.map,
                 path: self.path,
                 scopes: self.scopes,
@@ -513,7 +517,11 @@ impl<'a> Parser<'a> {
                             }
                             ('#', Some(Token { kind: '{', .. })) => {
                                 self.toks.next();
-                                comment.push_str(&self.parse_interpolation()?.to_css_string(span)?);
+                                comment.push_str(
+                                    &self
+                                        .parse_interpolation()?
+                                        .to_css_string(span, self.options.is_compressed())?,
+                                );
                                 continue;
                             }
                             (..) => comment.push(tok.kind),
@@ -541,7 +549,7 @@ impl<'a> Parser<'a> {
         let interpolation = self.parse_interpolation()?;
         Ok(match interpolation.node {
             Value::String(v, ..) => Cow::owned(v),
-            v => v.to_css_string(interpolation.span)?,
+            v => v.to_css_string(interpolation.span, self.options.is_compressed())?,
         })
     }
 
@@ -589,11 +597,8 @@ impl<'a> Parser<'a> {
                         self.toks.next();
                         #[allow(clippy::while_let_on_iterator)]
                         while let Some(tok) = self.toks.next() {
-                            if tok.kind == '*' {
-                                if let Some(Token { kind: '/', .. }) = self.toks.peek() {
-                                    self.toks.next();
-                                    break;
-                                }
+                            if tok.kind == '*' && self.consume_char_if_exists('/') {
+                                break;
                             }
                         }
                     }
@@ -613,7 +618,7 @@ impl<'a> Parser<'a> {
     }
 }
 
-impl<'a> Parser<'a> {
+impl<'a, 'b> Parser<'a, 'b> {
     fn parse_unknown_at_rule(&mut self, name: String) -> SassResult<Stmt> {
         if self.flags.in_function() {
             return Err(("This at-rule is not allowed here.", self.span_before).into());
@@ -621,36 +626,47 @@ impl<'a> Parser<'a> {
 
         let mut params = String::new();
         self.whitespace_or_comment();
-        if let Some(Token { kind: ';', .. }) | None = self.toks.peek() {
-            self.toks.next();
-            return Ok(Stmt::UnknownAtRule(Box::new(UnknownAtRule {
-                name,
-                super_selector: Selector::new(self.span_before),
-                params: String::new(),
-                body: Vec::new(),
-            })));
-        }
-        while let Some(tok) = self.toks.next() {
-            match tok.kind {
-                '{' => break,
-                '#' => {
+
+        loop {
+            match self.toks.peek() {
+                Some(Token { kind: '{', .. }) => {
+                    self.toks.next();
+                    break;
+                }
+                Some(Token { kind: ';', .. }) | Some(Token { kind: '}', .. }) | None => {
+                    self.consume_char_if_exists(';');
+                    return Ok(Stmt::UnknownAtRule(Box::new(UnknownAtRule {
+                        name,
+                        super_selector: Selector::new(self.span_before),
+                        has_body: false,
+                        params: params.trim().to_owned(),
+                        body: Vec::new(),
+                    })));
+                }
+                Some(Token { kind: '#', .. }) => {
+                    self.toks.next();
+
                     if let Some(Token { kind: '{', pos }) = self.toks.peek() {
-                        self.span_before = self.span_before.merge(*pos);
+                        self.span_before = self.span_before.merge(pos);
                         self.toks.next();
                         params.push_str(&self.parse_interpolation_as_string()?);
                     } else {
-                        params.push(tok.kind);
+                        params.push('#');
                     }
                     continue;
                 }
-                '\n' | ' ' | '\t' => {
+                Some(Token { kind: '\n', .. })
+                | Some(Token { kind: ' ', .. })
+                | Some(Token { kind: '\t', .. }) => {
                     self.whitespace();
                     params.push(' ');
                     continue;
                 }
-                _ => {}
+                Some(Token { kind, .. }) => {
+                    self.toks.next();
+                    params.push(kind);
+                }
             }
-            params.push(tok.kind);
         }
 
         let raw_body = self.parse_stmt()?;
@@ -664,9 +680,9 @@ impl<'a> Parser<'a> {
             }
         }
 
-        if !self.super_selectors.last().is_empty() {
+        if !self.super_selectors.last().as_selector_list().is_empty() {
             body = vec![Stmt::RuleSet {
-                selector: ExtendedSelector::new(self.super_selectors.last().clone().0),
+                selector: self.super_selectors.last().clone(),
                 body,
             }];
         }
@@ -677,6 +693,7 @@ impl<'a> Parser<'a> {
             name,
             super_selector: Selector::new(self.span_before),
             params: params.trim().to_owned(),
+            has_body: true,
             body,
         })))
     }
@@ -704,9 +721,9 @@ impl<'a> Parser<'a> {
             }
         }
 
-        if !self.super_selectors.last().is_empty() {
+        if !self.super_selectors.last().as_selector_list().is_empty() {
             body = vec![Stmt::RuleSet {
-                selector: ExtendedSelector::new(self.super_selectors.last().clone().0),
+                selector: self.super_selectors.last().clone(),
                 body,
             }];
         }
@@ -727,9 +744,16 @@ impl<'a> Parser<'a> {
             self.super_selectors.last().clone()
         } else {
             at_root_has_selector = true;
-            self.parse_selector(true, false, String::new())?.0
-        }
-        .resolve_parent_selectors(self.super_selectors.last(), false)?;
+            let selector = self
+                .parse_selector(true, false, String::new())?
+                .0
+                .resolve_parent_selectors(
+                    &self.super_selectors.last().clone().into_selector(),
+                    false,
+                )?;
+
+            self.extender.add_selector(selector.0, None)
+        };
 
         self.whitespace();
 
@@ -763,11 +787,27 @@ impl<'a> Parser<'a> {
             _ => Some(Ok(s)),
         })
         .collect::<SassResult<Vec<Stmt>>>()?;
-        let mut stmts = vec![Stmt::RuleSet {
-            selector: ExtendedSelector::new(at_rule_selector.0),
-            body: styles,
-        }];
-        stmts.extend(raw_stmts);
+
+        let stmts = if at_root_has_selector {
+            let mut body = styles;
+            body.extend(raw_stmts);
+
+            vec![Stmt::RuleSet {
+                body,
+                selector: at_rule_selector,
+            }]
+        } else {
+            if !styles.is_empty() {
+                return Err((
+                    "Found style at the toplevel inside @at-root.",
+                    self.span_before,
+                )
+                    .into());
+            }
+
+            raw_stmts
+        };
+
         Ok(stmts)
     }
 
@@ -780,9 +820,7 @@ impl<'a> Parser<'a> {
         //     return Err(("@extend may only be used within style rules.", self.span_before).into());
         // }
         let (value, is_optional) = Parser {
-            toks: &mut read_until_semicolon_or_closing_curly_brace(self.toks)?
-                .into_iter()
-                .peekmore(),
+            toks: &mut Lexer::new(read_until_semicolon_or_closing_curly_brace(self.toks)?),
             map: self.map,
             path: self.path,
             scopes: self.scopes,
@@ -818,20 +856,20 @@ impl<'a> Parser<'a> {
             }
 
             let compound = match complex.components.first() {
-                Some(ComplexSelectorComponent::Compound(c)) => c.clone(),
+                Some(ComplexSelectorComponent::Compound(c)) => c,
                 Some(..) | None => todo!(),
             };
             if compound.components.len() != 1 {
                 return Err((
                     format!(
                         "compound selectors may no longer be extended.\nConsider `@extend {}` instead.\nSee http://bit.ly/ExtendCompound for details.\n",
-                        compound.components.into_iter().map(|x| x.to_string()).collect::<Vec<String>>().join(", ")
+                        compound.components.iter().map(ToString::to_string).collect::<Vec<String>>().join(", ")
                     )
                 , self.span_before).into());
             }
 
             self.extender.add_extension(
-                super_selector.clone().0,
+                super_selector.clone().into_selector().0,
                 compound.components.first().unwrap(),
                 &extend_rule,
                 &None,
@@ -865,9 +903,9 @@ impl<'a> Parser<'a> {
             }
         }
 
-        if !self.super_selectors.last().is_empty() {
+        if !self.super_selectors.last().as_selector_list().is_empty() {
             body = vec![Stmt::RuleSet {
-                selector: ExtendedSelector::new(self.super_selectors.last().clone().0),
+                selector: self.super_selectors.last().clone(),
                 body,
             }];
         }
@@ -888,11 +926,15 @@ impl<'a> Parser<'a> {
             match tok.kind {
                 '{' => break,
                 '#' => {
-                    if let Some(Token { kind: '{', pos }) = self.toks.peek().copied() {
+                    if let Some(Token { kind: '{', pos }) = self.toks.peek() {
                         self.toks.next();
                         self.span_before = pos;
                         let interpolation = self.parse_interpolation()?;
-                        params.push_str(&interpolation.node.to_css_string(interpolation.span)?);
+                        params.push_str(
+                            &interpolation
+                                .node
+                                .to_css_string(interpolation.span, self.options.is_compressed())?,
+                        );
                         continue;
                     }
 
@@ -911,7 +953,7 @@ impl<'a> Parser<'a> {
     }
 }
 
-impl<'a> Parser<'a> {
+impl<'a, 'b> Parser<'a, 'b> {
     fn debug(&self, message: &Spanned<Cow<'a, str>>) {
         if self.options.quiet {
             return;
