@@ -7,7 +7,7 @@ use num_traits::Zero;
 
 use crate::{
     args::CallArgs,
-    common::{Op, QuoteKind},
+    common::{Identifier, Op, QuoteKind},
     error::SassResult,
     unit::Unit,
     value::{SassFunction, Value},
@@ -19,7 +19,7 @@ use super::super::Parser;
 pub(crate) enum HigherIntermediateValue {
     Literal(Value),
     /// A function that hasn't yet been evaluated
-    Function(SassFunction, CallArgs),
+    Function(SassFunction, CallArgs, Option<Spanned<Identifier>>),
     BinaryOp(Box<Self>, Op, Box<Self>),
     UnaryOp(Op, Box<Self>),
 }
@@ -30,19 +30,24 @@ impl HigherIntermediateValue {
     }
 }
 
-impl<'a> Parser<'a> {
-    fn call_function(&mut self, function: SassFunction, args: CallArgs) -> SassResult<Value> {
-        function.call(args, self)
+impl<'a, 'b> Parser<'a, 'b> {
+    fn call_function(
+        &mut self,
+        function: SassFunction,
+        args: CallArgs,
+        module: Option<Spanned<Identifier>>,
+    ) -> SassResult<Value> {
+        function.call(args, module, self)
     }
 }
 
-pub(crate) struct ValueVisitor<'a, 'b: 'a> {
-    parser: &'a mut Parser<'b>,
+pub(crate) struct ValueVisitor<'a, 'b: 'a, 'c> {
+    parser: &'a mut Parser<'b, 'c>,
     span: Span,
 }
 
-impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
-    pub fn new(parser: &'a mut Parser<'b>, span: Span) -> Self {
+impl<'a, 'b: 'a, 'c> ValueVisitor<'a, 'b, 'c> {
+    pub fn new(parser: &'a mut Parser<'b, 'c>, span: Span) -> Self {
         Self { parser, span }
     }
 
@@ -54,12 +59,71 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
             HigherIntermediateValue::Literal(v) => Ok(v),
             HigherIntermediateValue::BinaryOp(v1, op, v2) => self.bin_op(*v1, op, *v2, in_parens),
             HigherIntermediateValue::UnaryOp(op, val) => self.unary_op(op, *val, in_parens),
-            HigherIntermediateValue::Function(function, args) => {
-                self.parser.call_function(function, args)
+            HigherIntermediateValue::Function(function, args, module) => {
+                self.parser.call_function(function, args, module)
             }
         }
     }
 
+    fn bin_op_one_level(
+        &mut self,
+        val1: HigherIntermediateValue,
+        op: Op,
+        val2: HigherIntermediateValue,
+        in_parens: bool,
+    ) -> SassResult<HigherIntermediateValue> {
+        let val1 = self.unary(val1, in_parens)?;
+        let val2 = self.unary(val2, in_parens)?;
+
+        let val1 = match val1 {
+            HigherIntermediateValue::Literal(val1) => val1,
+            HigherIntermediateValue::BinaryOp(val1_1, val1_op, val1_2) => {
+                if val1_op.precedence() >= op.precedence() {
+                    return Ok(HigherIntermediateValue::BinaryOp(
+                        Box::new(self.bin_op_one_level(*val1_1, val1_op, *val1_2, in_parens)?),
+                        op,
+                        Box::new(val2),
+                    ));
+                }
+
+                return Ok(HigherIntermediateValue::BinaryOp(
+                    val1_1,
+                    val1_op,
+                    Box::new(self.bin_op_one_level(*val1_2, op, val2, in_parens)?),
+                ));
+            }
+            _ => unreachable!(),
+        };
+
+        let val2 = match val2 {
+            HigherIntermediateValue::Literal(val2) => val2,
+            HigherIntermediateValue::BinaryOp(val2_1, val2_op, val2_2) => {
+                todo!()
+            }
+            _ => unreachable!(),
+        };
+
+        let val1 = HigherIntermediateValue::Literal(val1);
+        let val2 = HigherIntermediateValue::Literal(val2);
+
+        Ok(HigherIntermediateValue::Literal(match op {
+            Op::Plus => self.add(val1, val2)?,
+            Op::Minus => self.sub(val1, val2)?,
+            Op::Mul => self.mul(val1, val2)?,
+            Op::Div => self.div(val1, val2, in_parens)?,
+            Op::Rem => self.rem(val1, val2)?,
+            Op::And => Self::and(val1, val2),
+            Op::Or => Self::or(val1, val2),
+            Op::Equal => Self::equal(val1, val2),
+            Op::NotEqual => Self::not_equal(val1, val2),
+            Op::GreaterThan => self.greater_than(val1, val2)?,
+            Op::GreaterThanEqual => self.greater_than_or_equal(val1, val2)?,
+            Op::LessThan => self.less_than(val1, val2)?,
+            Op::LessThanEqual => self.less_than_or_equal(val1, val2)?,
+            Op::Not => unreachable!(),
+        }))
+    }
+
     fn bin_op(
         &mut self,
         val1: HigherIntermediateValue,
@@ -68,19 +132,18 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
         in_parens: bool,
     ) -> SassResult<Value> {
         let mut val1 = self.unary(val1, in_parens)?;
-        let val2 = self.unary(val2, in_parens)?;
+        let mut val2 = self.unary(val2, in_parens)?;
 
-        if let HigherIntermediateValue::BinaryOp(val1_1, op2, val1_2) = val1 {
-            let in_parens = op != Op::Div || op2 != Op::Div;
-            if op2.precedence() >= op.precedence() {
-                val1 = HigherIntermediateValue::Literal(
-                    self.bin_op(*val1_1, op2, *val1_2, in_parens)?,
-                );
+        if let HigherIntermediateValue::BinaryOp(val1_1, val1_op, val1_2) = val1 {
+            let in_parens = op != Op::Div || val1_op != Op::Div;
+
+            return if val1_op.precedence() >= op.precedence() {
+                val1 = self.bin_op_one_level(*val1_1, val1_op, *val1_2, in_parens)?;
+                self.bin_op(val1, op, val2, in_parens)
             } else {
-                let val2 =
-                    HigherIntermediateValue::Literal(self.bin_op(*val1_2, op, val2, in_parens)?);
-                return self.bin_op(*val1_1, op2, val2, in_parens);
-            }
+                val2 = self.bin_op_one_level(*val1_2, op, val2, in_parens)?;
+                self.bin_op(*val1_1, val1_op, val2, in_parens)
+            };
         }
 
         Ok(match op {
@@ -122,14 +185,26 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                 Value::Dimension(Some(-n), u, should_divide)
             }
             Value::Dimension(None, u, should_divide) => Value::Dimension(None, u, should_divide),
-            v => Value::String(format!("-{}", v.to_css_string(self.span)?), QuoteKind::None),
+            v => Value::String(
+                format!(
+                    "-{}",
+                    v.to_css_string(self.span, self.parser.options.is_compressed())?
+                ),
+                QuoteKind::None,
+            ),
         })
     }
 
     fn unary_plus(&self, val: Value) -> SassResult<Value> {
         Ok(match val {
             v @ Value::Dimension(..) => v,
-            v => Value::String(format!("+{}", v.to_css_string(self.span)?), QuoteKind::None),
+            v => Value::String(
+                format!(
+                    "+{}",
+                    v.to_css_string(self.span, self.parser.options.is_compressed())?
+                ),
+                QuoteKind::None,
+            ),
         })
     }
 
@@ -146,8 +221,8 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
             HigherIntermediateValue::UnaryOp(op, val) => {
                 HigherIntermediateValue::Literal(self.unary_op(op, *val, in_parens)?)
             }
-            HigherIntermediateValue::Function(function, args) => {
-                HigherIntermediateValue::Literal(self.parser.call_function(function, args)?)
+            HigherIntermediateValue::Function(function, args, module) => {
+                HigherIntermediateValue::Literal(self.parser.call_function(function, args, module)?)
             }
             val => val,
         })
@@ -176,28 +251,36 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
             }
             Value::True | Value::False => match right {
                 Value::String(s, QuoteKind::Quoted) => Value::String(
-                    format!("{}{}", left.to_css_string(self.span)?, s),
+                    format!(
+                        "{}{}",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        s
+                    ),
                     QuoteKind::Quoted,
                 ),
                 _ => Value::String(
                     format!(
                         "{}{}",
-                        left.to_css_string(self.span)?,
-                        right.to_css_string(self.span)?
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
                     ),
                     QuoteKind::None,
                 ),
             },
             Value::Important => match right {
                 Value::String(s, ..) => Value::String(
-                    format!("{}{}", left.to_css_string(self.span)?, s),
+                    format!(
+                        "{}{}",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        s
+                    ),
                     QuoteKind::None,
                 ),
                 _ => Value::String(
                     format!(
                         "{}{}",
-                        left.to_css_string(self.span)?,
-                        right.to_css_string(self.span)?
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
                     ),
                     QuoteKind::None,
                 ),
@@ -205,7 +288,9 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
             Value::Null => match right {
                 Value::Null => Value::Null,
                 _ => Value::String(
-                    right.to_css_string(self.span)?.into_owned(),
+                    right
+                        .to_css_string(self.span, self.parser.options.is_compressed())?
+                        .into_owned(),
                     QuoteKind::None,
                 ),
             },
@@ -230,14 +315,34 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                         Value::Dimension(Some(num + num2.convert(&unit2, &unit)), unit, true)
                     }
                 }
-                Value::String(s, q) => Value::String(format!("{}{}{}", num, unit, s), q),
-                Value::Null => Value::String(format!("{}{}", num, unit), QuoteKind::None),
+                Value::String(s, q) => Value::String(
+                    format!(
+                        "{}{}{}",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit,
+                        s
+                    ),
+                    q,
+                ),
+                Value::Null => Value::String(
+                    format!(
+                        "{}{}",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit
+                    ),
+                    QuoteKind::None,
+                ),
                 Value::True
                 | Value::False
                 | Value::List(..)
                 | Value::Important
                 | Value::ArgList(..) => Value::String(
-                    format!("{}{}{}", num, unit, right.to_css_string(self.span)?),
+                    format!(
+                        "{}{}{}",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
                 Value::Map(..) | Value::FunctionRef(..) => {
@@ -251,7 +356,7 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                     return Err((
                         format!(
                             "Undefined operation \"{}{} + {}\".",
-                            num,
+                            num.inspect(),
                             unit,
                             right.inspect(self.span)?
                         ),
@@ -264,7 +369,11 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                 Value::String(s, q) => Value::String(format!("{}{}", c, s), q),
                 Value::Null => Value::String(c.to_string(), QuoteKind::None),
                 Value::List(..) => Value::String(
-                    format!("{}{}", c, right.to_css_string(self.span)?),
+                    format!(
+                        "{}{}",
+                        c,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
                 _ => {
@@ -281,17 +390,25 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
             },
             Value::String(text, quotes) => match right {
                 Value::String(text2, ..) => Value::String(text + &text2, quotes),
-                _ => Value::String(text + &right.to_css_string(self.span)?, quotes),
+                _ => Value::String(
+                    text + &right.to_css_string(self.span, self.parser.options.is_compressed())?,
+                    quotes,
+                ),
             },
             Value::List(..) | Value::ArgList(..) => match right {
-                Value::String(s, q) => {
-                    Value::String(format!("{}{}", left.to_css_string(self.span)?, s), q)
-                }
+                Value::String(s, q) => Value::String(
+                    format!(
+                        "{}{}",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        s
+                    ),
+                    q,
+                ),
                 _ => Value::String(
                     format!(
                         "{}{}",
-                        left.to_css_string(self.span)?,
-                        right.to_css_string(self.span)?
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
                     ),
                     QuoteKind::None,
                 ),
@@ -314,7 +431,10 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
         };
         Ok(match left {
             Value::Null => Value::String(
-                format!("-{}", right.to_css_string(self.span)?),
+                format!(
+                    "-{}",
+                    right.to_css_string(self.span, self.parser.options.is_compressed())?
+                ),
                 QuoteKind::None,
             ),
             v @ Value::Dimension(None, ..) => v,
@@ -344,7 +464,12 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                 | Value::True
                 | Value::False
                 | Value::ArgList(..) => Value::String(
-                    format!("{}{}-{}", num, unit, right.to_css_string(self.span)?),
+                    format!(
+                        "{}{}-{}",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
                 Value::Map(..) | Value::FunctionRef(..) => {
@@ -358,7 +483,7 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                     return Err((
                         format!(
                             "Undefined operation \"{}{} - {}\".",
-                            num,
+                            num.inspect(),
                             unit,
                             right.inspect(self.span)?
                         ),
@@ -366,7 +491,14 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                     )
                         .into())
                 }
-                Value::Null => Value::String(format!("{}{}-", num, unit), QuoteKind::None),
+                Value::Null => Value::String(
+                    format!(
+                        "{}{}-",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit
+                    ),
+                    QuoteKind::None,
+                ),
             },
             Value::Color(c) => match right {
                 Value::String(s, q) => {
@@ -385,32 +517,45 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                         .into())
                 }
                 _ => Value::String(
-                    format!("{}-{}", c, right.to_css_string(self.span)?),
+                    format!(
+                        "{}-{}",
+                        c,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
             },
             Value::String(..) => Value::String(
                 format!(
                     "{}-{}",
-                    left.to_css_string(self.span)?,
-                    right.to_css_string(self.span)?
+                    left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                    right.to_css_string(self.span, self.parser.options.is_compressed())?
                 ),
                 QuoteKind::None,
             ),
             _ => match right {
                 Value::String(s, q) => Value::String(
-                    format!("{}-{}{}{}", left.to_css_string(self.span)?, q, s, q),
+                    format!(
+                        "{}-{}{}{}",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        q,
+                        s,
+                        q
+                    ),
                     QuoteKind::None,
                 ),
                 Value::Null => Value::String(
-                    format!("{}-", left.to_css_string(self.span)?),
+                    format!(
+                        "{}-",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
                 _ => Value::String(
                     format!(
                         "{}-{}",
-                        left.to_css_string(self.span)?,
-                        right.to_css_string(self.span)?
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
                     ),
                     QuoteKind::None,
                 ),
@@ -448,7 +593,7 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                     return Err((
                         format!(
                             "Undefined operation \"{}{} * {}\".",
-                            num,
+                            num.inspect(),
                             unit,
                             right.inspect(self.span)?
                         ),
@@ -487,7 +632,10 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
         };
         Ok(match left {
             Value::Null => Value::String(
-                format!("/{}", right.to_css_string(self.span)?),
+                format!(
+                    "/{}",
+                    right.to_css_string(self.span, self.parser.options.is_compressed())?
+                ),
                 QuoteKind::None,
             ),
             Value::Dimension(None, ..) => todo!(),
@@ -537,24 +685,50 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                         }
                     } else {
                         Value::String(
-                            format!("{}{}/{}{}", num, unit, num2, unit2),
+                            format!(
+                                "{}{}/{}{}",
+                                num.to_string(self.parser.options.is_compressed()),
+                                unit,
+                                num2.to_string(self.parser.options.is_compressed()),
+                                unit2
+                            ),
                             QuoteKind::None,
                         )
                     }
                 }
-                Value::String(s, q) => {
-                    Value::String(format!("{}{}/{}{}{}", num, unit, q, s, q), QuoteKind::None)
-                }
+                Value::String(s, q) => Value::String(
+                    format!(
+                        "{}{}/{}{}{}",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit,
+                        q,
+                        s,
+                        q
+                    ),
+                    QuoteKind::None,
+                ),
                 Value::List(..)
                 | Value::True
                 | Value::False
                 | Value::Important
                 | Value::Color(..)
                 | Value::ArgList(..) => Value::String(
-                    format!("{}{}/{}", num, unit, right.to_css_string(self.span)?),
+                    format!(
+                        "{}{}/{}",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
+                    QuoteKind::None,
+                ),
+                Value::Null => Value::String(
+                    format!(
+                        "{}{}/",
+                        num.to_string(self.parser.options.is_compressed()),
+                        unit
+                    ),
                     QuoteKind::None,
                 ),
-                Value::Null => Value::String(format!("{}{}/", num, unit), QuoteKind::None),
                 Value::Map(..) | Value::FunctionRef(..) => {
                     return Err((
                         format!("{} isn't a valid CSS value.", right.inspect(self.span)?),
@@ -580,7 +754,11 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                         .into())
                 }
                 _ => Value::String(
-                    format!("{}/{}", c, right.to_css_string(self.span)?),
+                    format!(
+                        "{}/{}",
+                        c,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
             },
@@ -596,7 +774,13 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
                 | Value::Color(..)
                 | Value::List(..)
                 | Value::ArgList(..) => Value::String(
-                    format!("{}{}{}/{}", q1, s1, q1, right.to_css_string(self.span)?),
+                    format!(
+                        "{}{}{}/{}",
+                        q1,
+                        s1,
+                        q1,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
                 Value::Null => Value::String(format!("{}{}{}/", q1, s1, q1), QuoteKind::None),
@@ -610,18 +794,27 @@ impl<'a, 'b: 'a> ValueVisitor<'a, 'b> {
             },
             _ => match right {
                 Value::String(s, q) => Value::String(
-                    format!("{}/{}{}{}", left.to_css_string(self.span)?, q, s, q),
+                    format!(
+                        "{}/{}{}{}",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        q,
+                        s,
+                        q
+                    ),
                     QuoteKind::None,
                 ),
                 Value::Null => Value::String(
-                    format!("{}/", left.to_css_string(self.span)?),
+                    format!(
+                        "{}/",
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?
+                    ),
                     QuoteKind::None,
                 ),
                 _ => Value::String(
                     format!(
                         "{}/{}",
-                        left.to_css_string(self.span)?,
-                        right.to_css_string(self.span)?
+                        left.to_css_string(self.span, self.parser.options.is_compressed())?,
+                        right.to_css_string(self.span, self.parser.options.is_compressed())?
                     ),
                     QuoteKind::None,
                 ),
