@@ -1,12 +1,6 @@
 use codemap::Span;
 
-use crate::{
-    common::unvendor,
-    error::SassResult,
-    parse::Parser,
-    utils::{is_name, is_name_start, read_until_closing_paren},
-    Token,
-};
+use crate::{common::unvendor, error::SassResult, parse::Parser, utils::is_name, Token};
 
 use super::{
     Attribute, Combinator, ComplexSelector, ComplexSelectorComponent, CompoundSelector, Namespace,
@@ -50,21 +44,21 @@ const SELECTOR_PSEUDO_CLASSES: [&str; 8] = [
 /// Pseudo-element selectors that take unadorned selectors as arguments.
 const SELECTOR_PSEUDO_ELEMENTS: [&str; 1] = ["slotted"];
 
-pub(crate) struct SelectorParser<'a, 'b> {
+pub(crate) struct SelectorParser<'a, 'b, 'c> {
     /// Whether this parser allows the parent selector `&`.
     allows_parent: bool,
 
     /// Whether this parser allows placeholder selectors beginning with `%`.
     allows_placeholder: bool,
 
-    parser: &'a mut Parser<'b>,
+    parser: &'a mut Parser<'b, 'c>,
 
     span: Span,
 }
 
-impl<'a, 'b> SelectorParser<'a, 'b> {
+impl<'a, 'b, 'c> SelectorParser<'a, 'b, 'c> {
     pub fn new(
-        parser: &'a mut Parser<'b>,
+        parser: &'a mut Parser<'b, 'c>,
         allows_parent: bool,
         allows_placeholder: bool,
         span: Span,
@@ -170,7 +164,7 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
                     }
                 }
                 Some(..) => {
-                    if !self.looking_at_identifier() {
+                    if !self.parser.looking_at_identifier() {
                         break;
                     }
                     components.push(ComplexSelectorComponent::Compound(
@@ -188,17 +182,14 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
             return Err(("expected selector.", self.span).into());
         }
 
-        Ok(ComplexSelector {
-            components,
-            line_break,
-        })
+        Ok(ComplexSelector::new(components, line_break))
     }
 
     fn parse_compound_selector(&mut self) -> SassResult<CompoundSelector> {
         let mut components = vec![self.parse_simple_selector(None)?];
 
         while let Some(Token { kind, .. }) = self.parser.toks.peek() {
-            if !is_simple_selector_start(*kind) {
+            if !is_simple_selector_start(kind) {
                 break;
             }
 
@@ -208,34 +199,6 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
         Ok(CompoundSelector { components })
     }
 
-    /// Returns whether the scanner is immediately before a plain CSS identifier.
-    ///
-    // todo: foward arg
-    /// If `forward` is passed, this looks that many characters forward instead.
-    ///
-    /// This is based on [the CSS algorithm][], but it assumes all backslashes
-    /// start escapes.
-    ///
-    /// [the CSS algorithm]: https://drafts.csswg.org/css-syntax-3/#would-start-an-identifier
-    fn looking_at_identifier(&mut self) -> bool {
-        match self.parser.toks.peek() {
-            Some(Token { kind, .. }) if is_name_start(*kind) || kind == &'\\' => return true,
-            Some(Token { kind: '-', .. }) => {}
-            Some(..) | None => return false,
-        }
-
-        match self.parser.toks.peek_forward(1) {
-            Some(Token { kind, .. }) if is_name_start(*kind) || kind == &'-' || kind == &'\\' => {
-                self.parser.toks.reset_cursor();
-                true
-            }
-            Some(..) | None => {
-                self.parser.toks.reset_cursor();
-                false
-            }
-        }
-    }
-
     fn looking_at_identifier_body(&mut self) -> bool {
         matches!(self.parser.toks.peek(), Some(t) if is_name(t.kind) || t.kind == '\\')
     }
@@ -323,10 +286,15 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
             if SELECTOR_PSEUDO_ELEMENTS.contains(&unvendored) {
                 selector = Some(Box::new(self.parse_selector_list()?));
                 self.parser.whitespace();
-                self.parser.expect_char(')')?;
             } else {
-                argument = Some(self.declaration_value()?.into_boxed_str());
+                argument = Some(
+                    self.parser
+                        .declaration_value(true, false, true)?
+                        .into_boxed_str(),
+                );
             }
+
+            self.parser.expect_char(')')?;
         } else if SELECTOR_PSEUDO_CLASSES.contains(&unvendored) {
             selector = Some(Box::new(self.parse_selector_list()?));
             self.parser.whitespace();
@@ -349,11 +317,14 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
             argument = Some(this_arg.into_boxed_str());
         } else {
             argument = Some(
-                self.declaration_value()?
+                self.parser
+                    .declaration_value(true, false, true)?
                     .trim_end()
                     .to_owned()
                     .into_boxed_str(),
             );
+
+            self.parser.expect_char(')')?;
         }
 
         Ok(SimpleSelector::Pseudo(Pseudo {
@@ -391,7 +362,7 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
 
         match self.parser.toks.peek() {
             Some(Token { kind: '*', pos }) => {
-                self.parser.span_before = self.parser.span_before.merge(*pos);
+                self.parser.span_before = self.parser.span_before.merge(pos);
                 self.parser.toks.next();
                 if let Some(Token { kind: '|', .. }) = self.parser.toks.peek() {
                     self.parser.toks.next();
@@ -409,7 +380,7 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
                 return Ok(SimpleSelector::Universal(Namespace::None));
             }
             Some(Token { kind: '|', pos }) => {
-                self.parser.span_before = self.parser.span_before.merge(*pos);
+                self.parser.span_before = self.parser.span_before.merge(pos);
                 self.parser.toks.next();
                 match self.parser.toks.peek() {
                     Some(Token { kind: '*', .. }) => {
@@ -527,16 +498,6 @@ impl<'a, 'b> SelectorParser<'a, 'b> {
         Ok(buf)
     }
 
-    fn declaration_value(&mut self) -> SassResult<String> {
-        // todo: this consumes the closing paren
-        let mut tmp = read_until_closing_paren(self.parser.toks)?;
-        if let Some(Token { kind: ')', .. }) = tmp.pop() {
-        } else {
-            return Err(("expected \")\".", self.span).into());
-        }
-        Ok(tmp.into_iter().map(|t| t.kind).collect::<String>())
-    }
-
     fn expect_identifier(&mut self, s: &str) -> SassResult<()> {
         let mut ident = self.parser.parse_identifier_no_interpolation(false)?.node;
         ident.make_ascii_lowercase();
