@@ -1,13 +1,11 @@
 pub(crate) use chars::*;
 pub(crate) use comment_whitespace::*;
 pub(crate) use number::*;
-pub(crate) use peek_until::*;
 pub(crate) use read_until::*;
 pub(crate) use strings::*;
 
 mod chars;
 mod comment_whitespace;
 mod number;
-mod peek_until;
 mod read_until;
 mod strings;
