@@ -1,8 +1,7 @@
 use std::{
     cmp::Ordering,
     convert::{From, TryFrom},
-    fmt::{self, Display, Write},
-    mem,
+    fmt, mem,
     ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign},
 };
 
@@ -20,12 +19,29 @@ mod integer;
 
 const PRECISION: usize = 10;
 
-#[derive(Clone, Eq, PartialEq)]
+#[derive(Clone)]
 pub(crate) enum Number {
     Small(Rational64),
     Big(Box<BigRational>),
 }
 
+impl PartialEq for Number {
+    fn eq(&self, other: &Self) -> bool {
+        match (self, other) {
+            (Number::Small(val1), Number::Small(val2)) => val1 == val2,
+            (Number::Big(val1), val2 @ Number::Small(..)) => {
+                **val1 == val2.clone().into_big_rational()
+            }
+            (val1 @ Number::Small(..), Number::Big(val2)) => {
+                val1.clone().into_big_rational() == **val2
+            }
+            (Number::Big(val1), Number::Big(val2)) => val1 == val2,
+        }
+    }
+}
+
+impl Eq for Number {}
+
 impl Number {
     pub const fn new_small(val: Rational64) -> Number {
         Number::Small(val)
@@ -35,6 +51,17 @@ impl Number {
         Number::Big(Box::new(val))
     }
 
+    fn into_big_rational(self) -> BigRational {
+        match self {
+            Number::Small(small) => {
+                let tuple: (i64, i64) = small.into();
+
+                BigRational::new_raw(BigInt::from(tuple.0), BigInt::from(tuple.1))
+            }
+            Number::Big(big) => *big,
+        }
+    }
+
     pub fn to_integer(&self) -> Integer {
         match self {
             Self::Small(val) => Integer::Small(val.to_integer()),
@@ -112,7 +139,7 @@ impl Number {
     }
 
     #[allow(clippy::cast_precision_loss)]
-    fn as_float(self) -> Option<f64> {
+    pub fn as_float(self) -> Option<f64> {
         Some(match self {
             Number::Small(n) => ((*n.numer() as f64) / (*n.denom() as f64)),
             Number::Big(n) => ((n.numer().to_f64()?) / (n.denom().to_f64()?)),
@@ -131,6 +158,12 @@ impl Number {
         )?)))
     }
 
+    pub fn log(self, base: Number) -> Option<Self> {
+        Some(Number::Big(Box::new(BigRational::from_float(
+            self.as_float()?.log(base.as_float()?),
+        )?)))
+    }
+
     pub fn pow(self, exponent: Self) -> Option<Self> {
         Some(Number::Big(Box::new(BigRational::from_float(
             self.as_float()?.powf(exponent.as_float()?),
@@ -149,6 +182,7 @@ impl Number {
 
     /// Invariants: `from.comparable(&to)` must be true
     pub fn convert(self, from: &Unit, to: &Unit) -> Self {
+        debug_assert!(from.comparable(to));
         self * UNIT_CONVERSION_TABLE[to][from].clone()
     }
 }
@@ -312,8 +346,8 @@ from_smaller_integer!(u8);
 impl fmt::Debug for Number {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Self::Small(..) => write!(f, "Number::Small( {} )", self),
-            Self::Big(..) => write!(f, "Number::Big( {} )", self),
+            Self::Small(..) => write!(f, "Number::Small( {} )", self.to_string(false)),
+            Self::Big(..) => write!(f, "Number::Big( {} )", self.to_string(false)),
         }
     }
 }
@@ -354,16 +388,24 @@ impl ToPrimitive for Number {
     }
 }
 
-impl Display for Number {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+impl Number {
+    pub(crate) fn inspect(&self) -> String {
+        self.to_string(false)
+    }
+
+    pub(crate) fn to_string(&self, is_compressed: bool) -> String {
         let mut whole = self.to_integer().abs();
         let has_decimal = self.is_decimal();
         let mut frac = self.abs().fract();
         let mut dec = String::with_capacity(if has_decimal { PRECISION } else { 0 });
+
+        let mut buf = String::new();
+
         if has_decimal {
             for _ in 0..(PRECISION - 1) {
                 frac *= 10_i64;
-                write!(dec, "{}", frac.to_integer())?;
+                dec.push_str(&frac.to_integer().to_string());
+
                 frac = frac.fract();
                 if frac.is_zero() {
                     break;
@@ -397,20 +439,25 @@ impl Display for Number {
                         }
                     }
                 } else {
-                    write!(dec, "{}", end)?;
+                    dec.push_str(&end.to_string());
                 }
             }
         }
 
         if self.is_negative() && (!whole.is_zero() || !dec.is_empty()) {
-            f.write_char('-')?;
+            buf.push('-');
         }
-        write!(f, "{}", whole)?;
+
+        if !(whole.is_zero() && is_compressed) {
+            buf.push_str(&whole.to_string());
+        }
+
         if !dec.is_empty() {
-            f.write_char('.')?;
-            write!(f, "{}", dec)?;
+            buf.push('.');
+            buf.push_str(&dec);
         }
-        Ok(())
+
+        buf
     }
 }
 
@@ -721,7 +768,6 @@ impl Rem for Number {
     fn rem(self, other: Self) -> Self {
         match self {
             Self::Small(val1) => match other {
-                // todo: checked_rem for ratio?
                 Self::Small(val2) => {
                     let tuple1: (i64, i64) = val1.into();
                     let tuple2: (i64, i64) = val2.into();
